/*
 * Copyright (c) @ Regents of the University of California.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 * 	This product includes software developed by the MASH Research
 * 	Group at the University of California Berkeley.
 * 4. Neither the name of the University nor of the Research Group may be
 *    used to endorse or promote products derived from this software without
 *    specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * @(#) $Header: /usr/src/mash/repository/mash/mash-1/archive/mb-archive.h,v 1.3 1997/12/20 08:48:03 tecklee Exp $
 */


#ifndef MASH_MB_BUFFER_H
#define MASH_MB_BUFFER_H


#include "archive/timeval.h"
#include "mb/mb-time.h"

class DataBuffer {
public:
	DataBuffer() : pb(NULL), size(0) { }
	~DataBuffer() { free(); }
	Bool alloc(int newSize) { 
		if (newSize > size) pb = (Byte*) realloc(pb, newSize);
		if (pb==NULL) return FALSE;
		size = newSize;
		return TRUE;
	}
	void free() {
		if (pb!=NULL) {
			::free(pb);
			pb = NULL;
			size = 0;
		}
	}

	Byte *pb;
	int  size;
};


inline u_int32_t logical2mb(u_int32_t sec, u_int32_t usec)
{
	return sec*100 + usec/10000;
}


inline u_int32_t logical2mb(timeval tv)
{
	return logical2mb(tv.tv_sec, tv.tv_usec);
}


inline timeval mb2logical_units(u_int32_t ts)
{
	timeval tv;
	tv.tv_sec = ts / 100;
	tv.tv_usec = (ts % 100) * 10000;
	return tv;
}


inline timeval mb2logical(u_int32_t ts, timeval reference)
{
	u_int32_t refMB = logical2mb(reference);
	timeval diff;
	
	if (refMB <= ts) {
		// this case will happen if sender and receiver
		// clocks are not synchronized
		diff = mb2logical_units(ts - refMB);
		return (reference + diff);
	}
	else {
		diff = mb2logical_units(refMB - ts);
		return (reference - diff);
	}
}



#endif /* MASH_MB_BUFFER_H */
