
#include "tclcl.h"
#include "net.h"
#include "iohandler.h"
#ifdef WIN32
#include <io.h>
#define close closesocket
#else
#include <sys/types.h>
#include <sys/socket.h>
#endif


class UDPChannel : public SessionHandler, public TclObject {
    public:
	UDPChannel(int mtu);
	~UDPChannel();
	virtual void recv(DataHandler* dh);
	virtual int command(int argc, const char*const* argv);

    protected:
	DataHandler dh_;
	u_char* pktbuf_;
	char* cmdbuf_;
	int mtu_;

	virtual void recv(CtrlHandler*) {}
	virtual void announce(CtrlHandler*) {}
};

static class UDPChannelClass : public TclClass {
    public:
	UDPChannelClass() : TclClass("UDPChannel") {}
	TclObject* create(int, const char*const* argv) {
		return (new UDPChannel(atoi(argv[4])));
	}
} udp_channel_class;

UDPChannel::UDPChannel(int mtu) : mtu_(mtu)
{
	pktbuf_ = new u_char[2*mtu];
	cmdbuf_ = new char[2*mtu];
	dh_.manager(this);
}

UDPChannel::~UDPChannel()
{
	delete[] pktbuf_;
	delete[] cmdbuf_;
}

void
UDPChannel::recv(DataHandler* dh)
{
	u_int32_t addr; int fromPort;
	int cc = dh->recv(pktbuf_, 2*mtu_, addr, fromPort);
	if (cc < 0) {
		perror("recv");
		return;
	}
	if (cc == 0)
		return;

	Tcl& tcl = Tcl::instance();
	u_char* s = pktbuf_;
	s[cc] = '\0';
	sprintf(cmdbuf_, "%s recv %s %d {%s} %d", name(), intoa(addr), 
		fromPort, s, cc);
	tcl.eval(cmdbuf_);
}

int
UDPChannel::command(int argc, const char*const* argv)
{
	if (argc == 3) {
		if (strcmp(argv[1], "network") == 0) {
			Network* net = (Network*) TclObject::lookup(argv[2]);
			dh_.net(net);
			return (TCL_OK);
		} else if (strcmp(argv[1], "send") == 0) {
			int len = strlen(argv[2]);
			dh_.send((u_char*)argv[2], len);
			return (TCL_OK);
		}
	}
	return (TclObject::command(argc, argv));
}
