#
# Grouping Sources
#
Class GS

GS public init { prefops num_cluts } {
	$self instvar prefops_ num_cluts_
	set prefops_ $prefops
	set num_cluts_ $num_cluts
}

GS public get_clutmean { recv_id i } {
	$self instvar prefops_ cluts_

	set sum [$prefops_ zero]
	foreach src_id $cluts_(src,$i) {
		set sum [$prefops_ add $sum [$prefops_ get_pv $recv_id $src_id]]
	}
	set mean [$prefops_ scale $sum [expr 1 / double([llength $cluts_(src,$i)])]]
	return $mean
}

GS public get_dist_src_clutmean { src_id i } {
	$self instvar prefops_

	set sum 0
	foreach recv_id [$prefops_ get_recvlist] {
		set clutmean [$self get_clutmean $recv_id $i]
		set pv [$prefops_ get_pv $recv_id $src_id]
		set diff [$prefops_ subtract $pv $clutmean]
		set abs [$prefops_ abs $diff]
		set sum [$prefops_ add_s $sum $abs]
	}
	return $sum
}

GS public insert { src_id i } {
	$self instvar cluts_

	if ![info exists cluts_(src,$i)] {
		set cluts_(src,$i) [list $src_id]
	} else {
		set cluts_(src,$i) [lappend cluts_(src,$i) $src_id]
	}
}

GS public insert_src { src_id i } {
	$self instvar cluts_

	if ![info exists cluts_(src,$i)] {
		set cluts_(src,$i) [list $src_id]
	} else {
		set cluts_(src,$i) [lappend cluts_(src,$i) $src_id]
	}
}

GS public delete_src { src_id } {
	$self instvar cluts_ num_cluts_

	for {set i 1} {$i <= $num_cluts_} {incr i} {
		if { [info exists cluts_(src,$i)] && [lsearch $cluts_(src,$i) $src_id] != -1 } {
			set cluts_(src,$i) [lremove $cluts_(src,$i) $src_id]
			if { $cluts_(src,$i) == "" } {
				unset cluts_(src,$i)
			}
		}
	}
}

GS public insert_recv { recv_id lpv } {
	$self instvar prefops_
	$prefops_ set_pv $recv_id $lpv
}

GS public remove { src_id i } {
	$self instvar cluts_
	set cluts_(src,$i) [lremove $cluts_(src,$i) $src_id]
}

GS public move { src_id i j } {
	$self instvar cluts_
	$self remove $src_id $i
	set cluts_(src,$j) [lappend cluts_(src,$j) $src_id]
}

GS public get_size { } {
	$self instvar cluts_ num_cluts_
	set size 0
	for {set i 1} {$i <= $num_cluts_} {incr i} {
		if [info exists cluts_(src,$i)] {
			incr size [llength $cluts_(src,$i)]
		}
	}
	return $size
}

GS public get_num_moves { orig } {
	$self instvar cluts_ num_cluts_
	set num_moves 0
	for {set i 1} {$i <= num_cluts_} {incr i} {
		set c [lindex $orig [expr $i - 1]]
		foreach s $c {
			if { [lsearch $cluts_(src,$i) $s] == -1 } {
				incr num_moves
			}
		}
	}
	return $num_moves
}

GS public clear { } {
	$self instvar cluts_ num_cluts_

	for {set i 1} {$i <= $num_cluts_} {incr i} {
		if [info exists cluts_(src,$i)] {
			unset cluts_(src,$i)
		}
	}
}

GS public set_cluts { cluts } {
	$self instvar cluts_
	$self clear
	array set cluts_ $cluts
}

GS public set_default_cluts { l } {
	$self instvar cluts_ num_cluts_
	$self clear

	for {set i 1} {$i <= $num_cluts_} {incr i} {
		set cluts_(src,$i) [lindex $l $i]
	}
}

GS public get_cluts { } {
	$self instvar cluts_
	if [info exist cluts_] {
		return [array get cluts_]
	} 
}

GS public print { } {
	$self instvar cluts_ num_cluts_

	for {set i 1} {$i <= $num_cluts_} {incr i} {
		if [info exists cluts_(src,$i)] {
			puts "GS $i: $cluts_(src,$i)"
		}
	}
	#puts "Cost: [$self get_cost 0.001]"
}

#
# Implements k-means algorithm
#
Class GS/Kmeans -superclass GS

#
# Insert a new source into the system.
#
GS/Kmeans public insert_src { src_id } {
	$self instvar prefops_ num_cluts_ cluts_

	# step 0: if there is an empty cluster, insert src there and stop
	for {set i 1} {$i <= $num_cluts_} {incr i} {
		if ![info exists cluts_(src,$i)] {
			$self next $src_id $i
			return
		}
	}
	# step 1-4: find one has the minimum objective function with src inserted
	set min_dist 0xfffffff
	set min_clut -1
	for {set i 1} {$i <= $num_cluts_} {incr i} {
		set dist [$self get_dist_src_clutmean $src_id $i]
		if {$dist < $min_dist} {
			set min_dist $dist
			set min_clut $i
		} elseif {$dist == $min_dist} {
			# break ties by the cluster with smaller number of sources
			# or if the same the cluster with a smaller id
			if {[llength $cluts_(src,$i)] < [llength $cluts_(src,$min_clut)]} {
				set min_dist $dist
				set min_clut $i
			}
		}
	}
	$self insert $src_id $min_clut
	return $min_clut
}

#
# Adaptation process.
#
GS/Kmeans public adapt { thresh } {
	$self instvar cluts_ num_cluts_ prefops_

	# step 0: check whether current partition is acceptable
	puts stderr "CURRENT"
	$self print
	set cost [$prefops_ get_cost $self]
	if [$prefops_ acceptable $cost $thresh] {
		return $cost
	} 

	# step 1: check whether switching can reduce cost to acceptable 
	set copy [$self switch]
	
	# step 2: if switching can achieve threshold, use that partition
	puts stderr "SWITCHING"
	$self print
	set cost [$prefops_ get_cost $copy]
	if [$prefops_ acceptable $cost $thresh] {
		$self set_w_copy $copy
		return $cost
	}

	# step 3: if switching cannot achieve threshold, cluster from scratch
	puts stderr "SCRATCH"
	set src_list [$prefops_ get_srclist]
	set src_list [rand_permutate $src_list]
	
	set copy [$self get_a_copy]
	$copy clear
	foreach src_id $src_list {
		$copy insert_src $src_id
	}
	set scratch_cost [$prefops_ get_cost $copy]
	set ans [$prefops_ compare_cost $scratch_cost $cost] 
	if { $ans < 0 } {
		$self set_w_copy $copy
		return $scratch_cost
	} elseif { $ans > 0 } {
		puts stderr "warning: starting from scratch is worse."
	} else {
		puts stderr "warning: cannot improve clustering."
	}
	return $cost
}

#
# Adapt through switching algorithm.
#
GS/Kmeans public switch { } {
	$self instvar num_cluts_ cluts_ prefops_

	set copy [$self get_a_copy]
	set num_moves 0
	# step 1: calculate the error in the current partition

	# step 2-3: for each src, calculate the increase in error by moving
	# the src from its current cluster to each of the other clusters
	# in the partition
	for {set i 1} {$i <= $num_cluts_} {incr i} {
	if [info exists cluts_(src,$i)] {
	foreach src_id $cluts_(src,$i) {
		set min_diff 0
		set size_now [llength $cluts_(src,$i)]
		if { $size_now > 1 } {
		for {set j 1} {$j <= $num_cluts_} {incr j} {
		if { $j != $i } {
			set dist_now [$copy get_dist_src_clutmean $src_id $i]
			set dist_move [$copy get_dist_src_clutmean $src_id $j]
			set size_move [llength [$copy set cluts_(src,$j)]]
			set diff [expr (($size_move * $dist_move * $dist_move) / ($size_move + 1)) \
					- (($size_now * $dist_now * $dist_now) / ($size_now - 1))]
			if { $diff < $min_diff } {
				set min_diff $diff
				set min_clut $j
			}
		}
	        }
		if { $min_diff < 0 } {
			# just change a copy for now
			$copy move $src_id $i $min_clut
			incr num_moves
		}
	        }
	}
        }
        }
	return $copy
}

GS/Kmeans public get_a_copy { } {
	$self instvar num_cluts_ cluts_ prefops_
	set copy [new GS/Kmeans $prefops_ $num_cluts_]
	set cluts [$self get_cluts]
	$copy set_cluts $cluts
	return $copy
}

GS/Kmeans public set_w_copy { copy } {
	$self instvar num_cluts_ cluts_
	$self clear
	set cluts [$copy get_cluts]
	$self set_cluts $cluts
}
