

# trace only certain classes -- tcl-side only!
#
# don't bother creating new instances -- just say 
#     import Trace
# in relevent files, and to put a trace msg, call
#     Trc flag "my msg"
# or 
#     Trc "my msg"
#<p>
# Indicate which flags to trace via: 
#    Trace on|off
#    Trace add FLAG ...
# and/or
#    Trace rm FLAG ...
#
# e.g., for class-level ctrl of method tracing, in instprocs put:
#    Trc $class "--> ${class}::$proc"
#
# Precedence is as follows: on/off first, then individual flags,
# 	but if onoff==1 and flags="", trace everything
#
Class Trace

Trace set flags ""
Trace set onoff 0

Trace proc add {args} {
	set f [Trace set flags]
	foreach a $args {
		if {[lsearch -exact $f $a]==-1} {
			lappend f $a
		}
	}
	Trace set flags $f
}

Trace proc rm {args} {
	set f [Trace set flags]
	foreach a $args {
		set idx [lsearch -exact $f $a]
		if {$idx != -1} {
			set f [lreplace $f $idx $idx]
		}
	}
	Trace set flags $f
}

Trace proc on {} {Trace set onoff 1}
Trace proc off {} {Trace set onoff 0}

proc Trc {flag {msg ""}} {
	if ![Trace set onoff] {return}
	set t [clock format [clock seconds] -format {%H:%M:%S}]
	if {$msg == ""} {
		puts "\[$t\] Trc: $flag" ;# `flag' is actually the msg
	} else {
		set f [Trace set flags]
		if {$f == "" || [lsearch $f $flag] != -1} {
			puts "\[$t\] Trc - $flag: $msg"
		}
	}
}
